<?php

/**
 * @file
 * Module admin page callbacks.
 */

//////////////////////////////////////////////////////////////////////////////
// kerberos_authentication settings

/**
 * Implements the settings page.
 *
 * @return
 *   The form structure.
 */
function kerberos_authentication_admin_settings() {
  $options_login_process = array(
    KERBEROS_AUTHENTICATION_AUTH_MIXED => t('Mixed mode. The Kerberos authentication is performed only if Drupal authentication fails'),
    KERBEROS_AUTHENTICATION_AUTH_EXCLUSIVED => t('Kerberos only')
  );
  $options_login_conflict = array(
    KERBEROS_AUTHENTICATION_CONFLICT_LOG => t('Disallow login and log the conflict'),
    KERBEROS_AUTHENTICATION_CONFLICT_RESOLVE => t('Associate local account with the Kerberos entry')
  );

  $form['system-options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Authentication mode'),
    '#description' => t('<strong>NOTE:</strong> These settings have no effect on Drupal user with uid 1. The admin account never uses Kerberos.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['system-options']['kerberos_authentication_login_process'] = array(
    '#type' => 'radios',
    '#title' => t('Choose authentication mode'),
    '#description' => t('Pick the mode based on the types of user accounts and other configuration decisions. If <i>Kerberos only</i> option is activated some UI modications will be applied.'),
    '#default_value' => KERBEROS_AUTHENTICATION_LOGIN_PROCESS,
    '#options' => $options_login_process,
    '#required' => TRUE,
  );
  $form['system-options']['kerberos_authentication_login_conflict'] = array(
    '#type' => 'radios',
    '#title' => t('Choose user conflict resolve procedure'),
    '#description' => t('Pick what should be done if the local Drupal account already exists with the same login name.'),
    '#default_value' => KERBEROS_AUTHENTICATION_LOGIN_CONFLICT,
    '#options' => $options_login_conflict,
    '#required' => TRUE,
  );

  $form['security-options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Security Options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['security-options']['kerberos_authentication_forget_passwords'] = array(
    '#type' => 'checkbox',
    '#title' => t('Do not store users\' passwords during sessions'),
    '#default_value' => KERBEROS_AUTHENTICATION_FORGET_PASSWORDS,
  );
  $form['security-options']['kerberos_authentication_sync_passwords'] = array(
    '#type' => 'checkbox',
    '#title' => t('Sync Kerberos password with the Drupal password'),
    '#default_value' => KERBEROS_AUTHENTICATION_SYNC_PASSWORDS,
    '#description' => t('If checked, then Kerberos and Drupal passwords will be syncronized. This might be useful if some other modules need to authenticate against the user password hash stored in Drupal and works only in Mixed mode. It might introduce security issues in the Mixed mode since after deletion of the Kerberos principle user still be able to login to Drupal with his password. If unsure, leave this unchecked.'),
  );
  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['reset'] = array(
    '#type'  => 'submit',
    '#value' => t('Reset to defaults'),
  );
  return $form;
}

/**
 * Submit hook for the settings form.
 */
function kerberos_authentication_admin_settings_submit($form, &$form_state) {
  $op = $form_state['clicked_button']['#value'];
  $values = $form_state['values'];
  switch ($op) {
    case t('Save configuration'):
      variable_set('kerberos_authentication_login_process', $values['kerberos_authentication_login_process']);
      variable_set('kerberos_authentication_login_conflict', $values['kerberos_authentication_login_conflict']);
      variable_set('kerberos_authentication_forget_passwords', $values['kerberos_authentication_forget_passwords']);
      variable_set('kerberos_authentication_sync_passwords', $values['kerberos_authentication_sync_passwords']);



      drupal_set_message(t('The configuration options have been saved.'));
      break;
    case t('Reset to defaults'):
      variable_del('kerberos_authentication_login_process');
      variable_del('kerberos_authentication_login_conflict');
      variable_del('kerberos_authentication_forget_passwords');
      variable_del('kerberos_authentication_sync_passwords');
      drupal_set_message(t('The configuration options have been reset to their default values.'));
      break;
  }

  // Rebuild the menu router.
  menu_rebuild();
}

/**
 * Implements the Kerberos realm list.
 *
 * @return
 *   The HTML table with the realm list.
 */
function kerberos_authentication_admin_list() {
  $form['list'] = array();
  $result = db_query("SELECT sid, status, realm, weight FROM {kerberos_authentication} ORDER BY weight");
  while ($row = db_fetch_object($result)) {
    $form['list'][$row->sid] = array(
      'realm' => array('#value' => $row->realm),
      'status' => array('#value' => $row->status),
      'weight' => array(
        '#type' => 'weight',
        '#name' => 'sid_'. $row->sid .'_weight',
        '#delta' => 10,
        '#default_value' => $row->weight,
      ),
    );
  }

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Submit hook for the servers list form.
 */
function kerberos_authentication_admin_list_submit($form, &$form_state) {
  $op = $form_state['clicked_button']['#value'];
  switch ($op) {
    case t('Save'):
      foreach ($form_state['clicked_button']['#post'] as $name => $val) {
        if (preg_match('/^sid_.*_weight/', $name)) {
          $sid = preg_replace(array('/^sid_/', '/_weight$/'), array('', ''), $name);
          db_query("UPDATE {kerberos_authentication} SET weight = %d WHERE sid = %d", $val, $sid);
        }
      }
      break;
  }
}

/**
 * Implements the Kerberos realm edit page.
 *
 * @param $form_state
 *   A form state array.
 * @param $op
 *   An operatin - add or edit.
 * @param $sid
 *   A Kerberos server ID.
 *
 * @return
 *   The form structure.
 */
function kerberos_authentication_admin_form(&$form_state, $op = NULL, $sid = NULL) {
  if ($op == "edit" && $sid) {
    $edit = db_fetch_array(db_query("SELECT * FROM {kerberos_authentication} WHERE sid = %d", $sid));
    $form['sid'] = array(
      '#type' => 'hidden',
      '#value' => $sid,
    );
  }


  $form['realm-settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Realm settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
    $form['realm-settings']['realm'] = array(
    '#type' => 'textfield',
    '#title' => t('Realm'),
    '#default_value' => $edit['realm'],
    '#size' => 30,
    '#maxlength' => 255,
  );
   $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Validate hook for the Kerberos realm form.
 */
function kerberos_authentication_admin_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  if (!isset($values['sid'])) {
    if (db_fetch_object(db_query("SELECT realm FROM {kerberos_authentication} WHERE realm = '%s'", $values['realm']))) {
      form_set_error('realm', t('A Kerberos config with the  realm %realm already exists.', array('%realm' => $values['realm'])));
    }
  }
}

/**
 * Submit hook for the Kerberos realm form.
 */
function kerberos_authentication_admin_form_submit($form, &$form_state) {
  $op = $form_state['clicked_button']['#value'];
  $values = $form_state['values'];
  switch ($op) {
    case t('Save configuration'):
      if (!isset($values['sid'])) {
        db_query("INSERT INTO {kerberos_authentication} (status, realm) VALUES (%d, '%s')", 1, $values['realm']);
        drupal_set_message(t('Kerberos configuration %realm has been added.', array('%realm' => $values['realm'])));
        watchdog('Kerberos Authentication', 'Kerberos configuration %realm has been added.', array('%realm' => $values['realm']));
      }
      else {
        db_query("UPDATE {kerberos_authentication} SET realm = '%s' WHERE sid = %d", $values['realm'], $values['sid']);
        drupal_set_message(t('Kerberos Configuration %realm has been updated.', array('%realm' => $values['realm'])));
        watchdog('Kerberos Authentication', 'Kerberos Configuration %realm has been updated.', array('%realm' => $values['realm']));
      }

      $form_state['redirect'] = 'admin/settings/kerberos/list';
      break;
  }
}

/**
 * De-activates the Kerberos realm.
 *
 * @param $sid
 *   A Kerberos server ID.
 *
 * @return
 */
function kerberos_authentication_admin_deactivate($sid) {
  $result = db_query("SELECT name from {kerberos_authentication} WHERE sid = %d", $sid);
  if ($row = db_fetch_object($result)) {
    db_query("UPDATE {kerberos_authentication} SET status = '0' WHERE sid = %d", $sid);
    drupal_set_message(t('Kerberos Realm %realm has been de-activated.', array('%realm' => $row->realm)));
    watchdog('Kerberos Authentication', 'Kerberos Realm %realm was de-activated.', array('%realm' => $row->realm));
  }
  drupal_goto('admin/settings/kerberos/list');
}

/**
 * Activates the Kerberos realm.
 *
 * @param $sid
 *   A Kerberos sID.
 *
 * @return
 */
function kerberos_authentication_admin_activate($sid) {
  $result = db_query("SELECT name from {kerberos_authentication} WHERE sid = %d", $sid);
  if ($row = db_fetch_object($result)) {
    db_query("UPDATE {kerberos_authentication} SET status = '1' WHERE sid = %d", $sid);
    drupal_set_message(t('Kerberos Realm %realm has been activated.', array('%realm' => $row->realm)));
    watchdog('Kerberos Authentication', 'Kerberos Realm %realm was activated.', array('%realm' => $row->realm));
  }
  drupal_goto('admin/settings/kerberos/list');
}

/**
 * Implements the Kerberos realm delete page.
 *
 * @param $form_state
 *   A form state array.
 * @param $sid
 *   A Kerberos sID.
 *
 * @return
 *   The form structure.
 */
function kerberos_authentication_admin_delete(&$form_state, $sid) {
  if ($row = db_fetch_object(db_query("SELECT name FROM {kerberos_authentication} WHERE sid = %d", $sid))) {
    $form['sid'] = array(
      '#type' => 'hidden',
      '#value' => $sid,
    );
    $form['realm'] = array(
      '#type' => 'hidden',
      '#value' => $row->realm,
    );
    return confirm_form(
      $form,
      t('Are you sure you want to delete the Kerberos realm named <em><strong>%realm</strong></em>?', array('%realm' => $row->realm)),
      'admin/settings/kerberos/list',
      t('<p>This action cannot be undone.</p>'),
      t('Delete'),
      t('Cancel')
    );
  }
  drupal_goto('admin/settings/kerberos/list');
}

/**
 * Submit hook for the Kerberos realm delete page.
 */
function kerberos_authentication_admin_delete_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['confirm'] && $values['sid']) {
    db_query("DELETE FROM {kerberos_authentication} WHERE sid = %d", $values['sid']);
    drupal_set_message(t('Kerberos Realm %realm has been deleted.', array('%realm' => $values['realm'])));
    watchdog('Kerberos Authentication', 'Kerberos Realm %realm has been deleted.', array('%realm' => $values['realm']));
  }
}

/**
 * Implements the Kerberos admin  page.
 *
 * @return
 *   The themed HTML page.
 */
function kerberos_authentication_admin_menu_block_page() {
  return theme('admin_block_content', system_admin_menu_block(menu_get_item()));
}

